#include "moldyn.h"
#include <math.h>
#include <stdlib.h>

#define calcicell(A, B, C) (1+((A)-1+mx)%mx+(((B)-1+my)%my+(((C)-1+mz)%mz)*my)*mx)

void force (float *potential, float *virial, float rx[], float ry[], float rz[], int head[], int list[], int natm)
{
   extern float sigma, rcut, vrcut, dvrc12, dvrcut;
   extern float *scrat;
   extern float cellix, celliy, celliz, sfx, sfy, sfz;
   extern int mx, my, mz;
   extern int map[3][3][3];
   float sigsq, rcutsq;
   float *fx=&scrat[FOR1];
   float *fy=&scrat[FOR2];
   float *fz=&scrat[FOR3];
   float rxi, ryi, rzi, fxi, fyi, fzi;
   float rxij, ryij, rzij, rijsq;
   float rij, sr2, sr6, vij, wij, fij, fxij, fyij, fzij;
   int i, icell, j, jcell;
   int x,y,z,xc,yc,zc;

   sigsq  = sigma*sigma;
   rcutsq = rcut*rcut;

   *potential = 0.0;
   *virial    = 0.0;

   float v = *virial;
   float p = *potential;

   #pragma omp parallel default(none) shared(v,p,fx,fy,fz,rx,ry,rz,head,list,map,natm,sigsq,rcutsq,rcut,vrcut,dvrc12,dvrcut,mx,my,mz,cellix,celliy,celliz,sfx,sfy,sfz)
   {

      #pragma omp for private(i)
      for(i=0;i<natm;++i)
      {
         *(fx+i) = 0.0;
         *(fy+i) = 0.0;
         *(fz+i) = 0.0;
      }

      #pragma omp for private(xc,yc,zc,rxi,ryi,rzi,fxi,fyi,fzi,rxij,ryij,rzij,rijsq,rij,sr2,sr6,vij,wij,fij,fxij,fyij,fzij,i,icell,j,jcell) reduction(+:v,p)
      for(icell=1; icell <= mx*my*mz; icell++)
      {
               i = head[icell-1];

               while (i>0)
               {
                  rxi = rx[i-1];
                  ryi = ry[i-1];
                  rzi = rz[i-1];

                  if (i <= natm)
                  {
                     fxi = fx[i-1];
                     fyi = fy[i-1];
                     fzi = fz[i-1];
                     j = list[i-1];

                     while (j>0)
                     {
                        rxij = rxi - rx[j-1];
                        ryij = ryi - ry[j-1];
                        rzij = rzi - rz[j-1];
                        rijsq = rxij*rxij + ryij*ryij + rzij*rzij;

                        if (rijsq < rcutsq)
                        {
                           rij        = (float) sqrt ((double)rijsq);
                           sr2        = sigsq/rijsq;
                           sr6        = sr2*sr2*sr2;
                           vij        = sr6*(sr6-1.0) - vrcut - dvrc12*(rij-rcut);
                           wij        = sr6*(sr6-0.5) + dvrcut*rij;
                           p         += vij;
                           v         += wij;
                           fij        = wij/rijsq;
                           fxij       = fij*rxij;
                           fyij       = fij*ryij;
                           fzij       = fij*rzij;
                           fxi       += fxij;
                           fyi       += fyij;
                           fzi       += fzij;
                           *(fx+j-1) -= fxij;
                           *(fy+j-1) -= fyij;
                           *(fz+j-1) -= fzij;
                        }
                        j = list[j-1];
                     }
                  }
                  for(xc=0;xc<3;xc++)
                  {
                     for(yc=0;yc<3;yc++)
                     {
                        for(zc=0;zc<3;zc++)
                        {
                           if (map[xc][yc][zc]==1)
                           {
                              int x = 1+(int)(cellix*(rx[i-1]*sfx+0.5));
                              int y = 1+(int)(celliy*(ry[i-1]*sfy+0.5));
                              int z = 1+(int)(celliz*(rz[i-1]*sfz+0.5));
                              int xcell = x + xc-1;
                              int ycell = y + yc-1;
                              int zcell = z + zc-1;

                              jcell = calcicell(xcell,ycell,zcell);
                              j = head[jcell-1];

                              if (i<=natm || j<=natm)
                              {
                                 while (j!=0)
                                 {
                                    rxij = rxi - rx[j-1];
                                    ryij = ryi - ry[j-1];
                                    rzij = rzi - rz[j-1];
                                    rijsq = rxij*rxij + ryij*ryij + rzij*rzij;

                                    if (rijsq < rcutsq)
                                    {
                                       rij        = (float) sqrt ((double)rijsq);
                                       sr2        = sigsq/rijsq;
                                       sr6        = sr2*sr2*sr2;
                                       vij        = (sr6*(sr6-1.0) - vrcut - dvrc12*(rij-rcut))*0.5;
                                       wij        = sr6*(sr6-0.5) + dvrcut*rij;
                                       fij        = wij/rijsq;
                                       wij       *= 0.5;
                                       fxij       = fij*rxij;
                                       fyij       = fij*ryij;
                                       fzij       = fij*rzij;

                                       if (i<=natm)
                                       {
                                          p        += vij;
                                          v        += wij;
                                          fxi      += fxij;
                                          fyi      += fyij;
                                          fzi      += fzij;
                                       }

                                       if (j<=natm)
                                       {
                                          p         += vij;
                                          v         += wij;
                                          *(fx+j-1) -= fxij;
                                          *(fy+j-1) -= fyij;
                                          *(fz+j-1) -= fzij;
                                       }
                                    }
                                    j = list[j-1];
                                 }
                              }
                           }
                        }
                     }
                  }

                  if (i<=natm)
                  {
                     *(fx+i-1) = fxi;
                     *(fy+i-1) = fyi;
                     *(fz+i-1) = fzi;
                  }
                  i = list[i-1];
               }
      }
      #pragma omp barrier
      /* End of forces loop */

      #pragma omp for nowait private(i)
      for (i=1;i<=natm;++i)
      {
         *(fx+i-1) *= 48.0;
         *(fy+i-1) *= 48.0;
         *(fz+i-1) *= 48.0;
      }
   }
   /* End of parallel section */

   *potential += p;
   *virial += v;

   *potential *= 4.0;
   *virial    *= 48.0/3.0;
}
